@@ -748,10 +748,11 @@ setMethod(
           segment_feasible  <- unique(o@theta_segment_index[o@shadow_test_feasible == TRUE])
           theta_is_feasible <- segment_of$final_theta_est %in% segment_feasible
           eligible_flag     <- getEligibleFlag(ineligible_flag, constants, !theta_is_feasible)
-          exposure_record   <- applyIncrement(exposure_record, segments_to_apply, segment_prob, theta_is_feasible, eligible_flag, o, exposure_constants, constants)
-          exposure_record   <- applyIncrementVisitedSegments(exposure_record, segment_prob, segment_of$visited, ineligible_flag_in_segment, o, exposure_constants, constants)
-          exposure_record   <- applyAcceleration(exposure_record, exposure_constants, constants)
-          exposure_record   <- applyClip(exposure_record, constants)
+          exposure_record   <- incrementExamineeCount(exposure_record, segments_to_apply, segment_prob, theta_is_feasible, exposure_constants)
+          exposure_record   <- incrementAdministrationCount(exposure_record, segments_to_apply, segment_prob, eligible_flag, o, exposure_constants, constants)
+          exposure_record   <- incrementAdministrationCountForVisitedSegments(exposure_record, segment_prob, segment_of$visited, ineligible_flag_in_segment, o, exposure_constants, constants)
+          exposure_record   <- updateEligibilityRates(exposure_record, exposure_constants, constants)
+          exposure_record   <- clipEligibilityRates(exposure_record, constants)
 
         } else if (exposure_control %in% c("BIGM")) {
 
@@ -759,10 +760,11 @@ setMethod(
           exposure_record   <- applyFading(exposure_record, segments_to_apply, exposure_constants, constants)
           segment_prob      <- 1
           eligible_flag     <- getEligibleFlag(ineligible_flag, constants, FALSE)
-          exposure_record   <- applyIncrement(exposure_record, segments_to_apply, segment_prob, FALSE, eligible_flag, o, exposure_constants, constants)
-          exposure_record   <- applyIncrementVisitedSegments(exposure_record, segment_prob, segment_of$visited, ineligible_flag_in_segment, o, exposure_constants, constants)
-          exposure_record   <- applyAcceleration(exposure_record, exposure_constants, constants)
-          exposure_record   <- applyClip(exposure_record, constants)
+          exposure_record   <- incrementExamineeCount(exposure_record, segments_to_apply, segment_prob, FALSE, exposure_constants)
+          exposure_record   <- incrementAdministrationCount(exposure_record, segments_to_apply, segment_prob, eligible_flag, o, exposure_constants, constants)
+          exposure_record   <- incrementAdministrationCountForVisitedSegments(exposure_record, segment_prob, segment_of$visited, ineligible_flag_in_segment, o, exposure_constants, constants)
+          exposure_record   <- updateEligibilityRates(exposure_record, exposure_constants, constants)
+          exposure_record   <- clipEligibilityRates(exposure_record, constants)
 
         } else if (exposure_control %in% c("BIGM-BAYESIAN")) {
 
@@ -770,10 +772,11 @@ setMethod(
           exposure_record   <- applyFading(exposure_record, segments_to_apply, exposure_constants, constants)
           segment_prob      <- getSegmentProb(current_theta$posterior_sample, exposure_constants)
           eligible_flag     <- getEligibleFlag(ineligible_flag, constants, FALSE)
-          exposure_record   <- applyIncrement(exposure_record, segments_to_apply, segment_prob, FALSE, eligible_flag, o, exposure_constants, constants)
-          exposure_record   <- applyIncrementVisitedSegments(exposure_record, segment_prob, segment_of$visited, ineligible_flag_in_segment, o, exposure_constants, constants)
-          exposure_record   <- applyAcceleration(exposure_record, exposure_constants, constants)
-          exposure_record   <- applyClip(exposure_record, constants)
+          exposure_record   <- incrementExamineeCount(exposure_record, segments_to_apply, segment_prob, FALSE, exposure_constants)
+          exposure_record   <- incrementAdministrationCount(exposure_record, segments_to_apply, segment_prob, eligible_flag, o, exposure_constants, constants)
+          exposure_record   <- incrementAdministrationCountForVisitedSegments(exposure_record, segment_prob, segment_of$visited, ineligible_flag_in_segment, o, exposure_constants, constants)
+          exposure_record   <- updateEligibilityRates(exposure_record, exposure_constants, constants)
+          exposure_record   <- clipEligibilityRates(exposure_record, constants)
 
         }
 
@@ -787,10 +790,6 @@ setMethod(
 
       }
 
-      if (config@audit_trail) {
-        plotAuditTrail()
-      }
-
       if (!is.null(session)) {
         shinyWidgets::updateProgressBar(session = session, id = "pb", value = j, total = constants$nj)
       } else {
